'use strict';

var fs$1 = require('fs');
var fs = require('fs/promises');
var path = require('path');
var readPkgUp = require('read-pkg-up');
var vite = require('vite');
var runtime = require('./runtime-CSusHM37.js');
var chalk = require('chalk');
function _interopDefaultCompat(e) {
  return e && typeof e === 'object' && 'default' in e ? e : {
    default: e
  };
}
var fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
var path__default = /*#__PURE__*/_interopDefaultCompat(path);
var readPkgUp__default = /*#__PURE__*/_interopDefaultCompat(readPkgUp);
var chalk__default = /*#__PURE__*/_interopDefaultCompat(chalk);
const debug$1 = runtime.debug.extend("static");
async function buildStaticFiles(options) {
  const {
    cwd,
    outputDir,
    sourceMap = false,
    minify = true,
    basePath,
    vite: extendViteConfig
  } = options;
  debug$1("Writing Sanity runtime files");
  await runtime.writeSanityRuntime({
    cwd,
    reactStrictMode: false,
    watch: false,
    basePath
  });
  debug$1("Resolving vite config");
  const mode = "production";
  let viteConfig = await runtime.getViteConfig({
    cwd,
    basePath,
    outputDir,
    minify,
    sourceMap,
    mode
  });
  if (extendViteConfig) {
    viteConfig = await runtime.extendViteConfigWithUserConfig({
      command: "build",
      mode
    }, viteConfig, extendViteConfig);
    viteConfig = runtime.finalizeViteConfig(viteConfig);
  }
  debug$1("Copying static files from /static to output dir");
  const staticPath = path__default.default.join(outputDir, "static");
  await copyDir(path__default.default.join(cwd, "static"), staticPath);
  debug$1("Writing favicons to output dir");
  const faviconBasePath = "".concat(basePath.replace(/\/+$/, ""), "/static");
  await writeFavicons(faviconBasePath, staticPath);
  debug$1("Bundling using vite");
  const bundle = await vite.build(viteConfig);
  debug$1("Bundling complete");
  if (Array.isArray(bundle) || !("output" in bundle)) {
    return {
      chunks: []
    };
  }
  const stats = [];
  bundle.output.forEach(chunk => {
    if (chunk.type !== "chunk") {
      return;
    }
    stats.push({
      name: chunk.name,
      modules: Object.entries(chunk.modules).map(_ref => {
        let [rawFilePath, chunkModule] = _ref;
        const filePath = rawFilePath.startsWith("\0") ? rawFilePath.slice("\0".length) : rawFilePath;
        return {
          name: path__default.default.isAbsolute(filePath) ? path__default.default.relative(cwd, filePath) : filePath,
          originalLength: chunkModule.originalLength,
          renderedLength: chunkModule.renderedLength
        };
      })
    });
  });
  return {
    chunks: stats
  };
}
async function copyDir(srcDir, destDir, skipExisting) {
  await fs__default.default.mkdir(destDir, {
    recursive: true
  });
  for (const file of await tryReadDir(srcDir)) {
    const srcFile = path__default.default.resolve(srcDir, file);
    if (srcFile === destDir) {
      continue;
    }
    const destFile = path__default.default.resolve(destDir, file);
    const stat = await fs__default.default.stat(srcFile);
    if (stat.isDirectory()) {
      await copyDir(srcFile, destFile, skipExisting);
    } else if (skipExisting) {
      await fs__default.default.copyFile(srcFile, destFile, fs$1.constants.COPYFILE_EXCL).catch(skipIfExistsError);
    } else {
      await fs__default.default.copyFile(srcFile, destFile);
    }
  }
}
async function tryReadDir(dir) {
  try {
    const content = await fs__default.default.readdir(dir);
    return content;
  } catch (err) {
    if (err.code === "ENOENT") {
      return [];
    }
    throw err;
  }
}
function skipIfExistsError(err) {
  if (err.code === "EEXIST") {
    return;
  }
  throw err;
}
async function writeFavicons(basePath, destDir) {
  var _a;
  const sanityPkgPath = (_a = await readPkgUp__default.default({
    cwd: __dirname
  })) == null ? void 0 : _a.path;
  const faviconsPath = sanityPkgPath ? path__default.default.join(path__default.default.dirname(sanityPkgPath), "static", "favicons") : void 0;
  if (!faviconsPath) {
    throw new Error("Unable to resolve `sanity` module root");
  }
  await fs__default.default.mkdir(destDir, {
    recursive: true
  });
  await copyDir(faviconsPath, destDir, true);
  await writeWebManifest(basePath, destDir);
  await fs__default.default.copyFile(path__default.default.join(destDir, "favicon.ico"), path__default.default.join(destDir, "..", "favicon.ico"));
}
async function writeWebManifest(basePath, destDir) {
  const content = JSON.stringify(runtime.generateWebManifest(basePath), null, 2);
  await fs__default.default.writeFile(path__default.default.join(destDir, "manifest.webmanifest"), content, "utf8").catch(skipIfExistsError);
}
function sanityBasePathRedirectPlugin(basePath) {
  return {
    name: "sanity/server/sanity-base-path-redirect",
    apply: "serve",
    configurePreviewServer(vitePreviewServer) {
      return () => {
        if (!basePath) {
          return;
        }
        vitePreviewServer.middlewares.use((req, res, next) => {
          if (req.url !== "/") {
            next();
            return;
          }
          res.writeHead(302, {
            Location: basePath
          });
          res.end();
        });
      };
    }
  };
}
const debug = runtime.debug.extend("preview");
async function startPreviewServer(options) {
  const {
    httpPort,
    httpHost,
    root,
    vite: extendViteConfig
  } = options;
  const startTime = Date.now();
  const indexPath = path__default.default.join(root, "index.html");
  let basePath;
  try {
    const index = await fs__default.default.readFile(indexPath, "utf8");
    basePath = tryResolveBasePathFromIndex(index);
  } catch (err) {
    if (err.code !== "ENOENT") {
      throw err;
    }
    const error = new Error("Could not find a production build in the '".concat(root, "' directory.\nTry building your studio app with 'sanity build' before starting the preview server."));
    error.name = "BUILD_NOT_FOUND";
    throw error;
  }
  const mode = "production";
  let previewConfig = {
    root,
    base: basePath || "/",
    plugins: [sanityBasePathRedirectPlugin(basePath)],
    configFile: false,
    preview: {
      port: httpPort,
      host: httpHost,
      strictPort: true
    },
    // Needed for vite to not serve `root/dist`
    build: {
      outDir: root
    },
    mode
  };
  if (extendViteConfig) {
    previewConfig = await runtime.extendViteConfigWithUserConfig({
      command: "serve",
      mode
    }, previewConfig, extendViteConfig);
  }
  debug("Creating vite server");
  const server = await vite.preview(previewConfig);
  const warn = server.config.logger.warn;
  const info = server.config.logger.info;
  const url = server.resolvedUrls.local[0];
  if (typeof basePath === "undefined") {
    warn('Could not determine base path from index.html, using "/" as default');
  } else if (basePath && basePath !== "/") {
    info("Using resolved base path from static build: ".concat(chalk__default.default.cyan(basePath)));
  }
  const startupDuration = Date.now() - startTime;
  info("Sanity Studio " + "using ".concat(chalk__default.default.cyan("vite@".concat(require("vite/package.json").version)), " ") + "ready in ".concat(chalk__default.default.cyan("".concat(Math.ceil(startupDuration), "ms")), " ") + "and running at ".concat(chalk__default.default.cyan(url), " (production preview mode)"));
  return {
    urls: server.resolvedUrls,
    close: () => new Promise((resolve, reject) => server.httpServer.close(err => err ? reject(err) : resolve()))
  };
}
function tryResolveBasePathFromIndex(index) {
  var _a;
  const basePath = (_a = index.match(/<script[^>]+src="(.*?)\/static\/sanity-/)) == null ? void 0 : _a[1];
  if (typeof basePath === "undefined") {
    return void 0;
  }
  return basePath === "" ? "/" : basePath;
}
exports.buildStaticFiles = buildStaticFiles;
exports.startPreviewServer = startPreviewServer;
//# sourceMappingURL=previewServer-DZ4NiF4u.js.map
