'use strict';

var isPlainObject = require('lodash/isPlainObject.js');
var oneline = require('oneline');
var worker_threads = require('worker_threads');
var getStudioWorkspaces = require('../../../_chunks/getStudioWorkspaces-BP-leiyc.js');
function _interopDefaultCompat(e) {
  return e && typeof e === 'object' && 'default' in e ? e : {
    default: e
  };
}
var isPlainObject__default = /*#__PURE__*/_interopDefaultCompat(isPlainObject);
var oneline__default = /*#__PURE__*/_interopDefaultCompat(oneline);
var __freeze = Object.freeze;
var __defProp = Object.defineProperty;
var __template = (cooked, raw) => __freeze(__defProp(cooked, "raw", {
  value: __freeze(raw || cooked.slice())
}));
var _a;
if (worker_threads.isMainThread || !worker_threads.parentPort) {
  throw new Error("This module must be run as a worker thread");
}
getGraphQLAPIsForked(worker_threads.parentPort);
async function getGraphQLAPIsForked(parent) {
  const {
    cliConfig,
    cliConfigPath,
    workDir
  } = worker_threads.workerData;
  const resolved = await resolveGraphQLApis({
    cliConfig,
    cliConfigPath,
    workDir
  });
  parent.postMessage(resolved);
}
async function resolveGraphQLApis(_ref) {
  let {
    cliConfig,
    cliConfigPath,
    workDir
  } = _ref;
  const workspaces = await getStudioWorkspaces.getStudioWorkspaces({
    basePath: workDir
  });
  const numSources = workspaces.reduce((count, workspace) => count + workspace.unstable_sources.length, 0);
  const multiSource = numSources > 1;
  const multiWorkspace = workspaces.length > 1;
  const hasGraphQLConfig = Boolean(cliConfig == null ? void 0 : cliConfig.graphql);
  if (workspaces.length === 0) {
    throw new Error("No studio configuration found");
  }
  if (numSources === 0) {
    throw new Error("No sources (project ID / dataset) configured");
  }
  if ((multiWorkspace || multiSource) && !hasGraphQLConfig) {
    throw new Error(oneline__default.default(_a || (_a = __template(["\n      Multiple workspaces/sources configured.\n      You must define an array of GraphQL APIs in ", "\n      and specify which workspace/source to use.\n    "])), cliConfigPath || "sanity.cli.js"));
  }
  if (!hasGraphQLConfig) {
    const {
      projectId,
      dataset,
      schema
    } = workspaces[0].unstable_sources[0];
    return [{
      schemaTypes: getStrippedSchemaTypes(schema),
      projectId,
      dataset
    }];
  }
  const apiDefs = validateCliConfig((cliConfig == null ? void 0 : cliConfig.graphql) || []);
  return resolveGraphQLAPIsFromConfig(apiDefs, workspaces);
}
function resolveGraphQLAPIsFromConfig(apiDefs, workspaces) {
  const resolvedApis = [];
  for (const apiDef of apiDefs) {
    const {
      workspace: workspaceName,
      source: sourceName
    } = apiDef;
    if (!workspaceName && workspaces.length > 1) {
      throw new Error("Must define `workspace` name in GraphQL API config when multiple workspaces are defined");
    }
    const workspace = !workspaceName && workspaces.length === 1 ? workspaces[0] : workspaces.find(space => space.name === (workspaceName || "default"));
    if (!workspace) {
      throw new Error('Workspace "'.concat(workspaceName || "default", '" not found'));
    }
    const source = !sourceName && workspace.unstable_sources.length === 1 ? workspace.unstable_sources[0] : workspace.unstable_sources.find(src => src.name === (sourceName || "default"));
    if (!source) {
      throw new Error('Source "'.concat(sourceName || "default", '" not found in workspace "').concat(workspaceName || "default", '"'));
    }
    resolvedApis.push({
      ...apiDef,
      dataset: source.dataset,
      projectId: source.projectId,
      schemaTypes: getStrippedSchemaTypes(source.schema)
    });
  }
  return resolvedApis;
}
function validateCliConfig(config) {
  let configPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "sanity.cli.js";
  if (!Array.isArray(config)) {
    throw new Error('"graphql" key in "'.concat(configPath, '" must be an array if defined'));
  }
  if (config.length === 0) {
    throw new Error('No GraphQL APIs defined in "'.concat(configPath, '"'));
  }
  return config;
}
function getStrippedSchemaTypes(schema) {
  const schemaDef = schema._original || {
    types: []
  };
  return schemaDef.types.map(type => stripType(type));
}
function stripType(input) {
  return strip(input);
}
function strip(input) {
  if (Array.isArray(input)) {
    return input.map(item => strip(item)).filter(item => typeof item !== "undefined");
  }
  if (isPlainishObject(input)) {
    return Object.keys(input).reduce((stripped, key) => {
      stripped[key] = strip(input[key]);
      return stripped;
    }, {});
  }
  return isBasicType(input) ? input : void 0;
}
function isPlainishObject(input) {
  return isPlainObject__default.default(input);
}
function isBasicType(input) {
  const type = typeof input;
  if (type === "boolean" || type === "number" || type === "string") {
    return true;
  }
  if (type !== "object") {
    return false;
  }
  return Array.isArray(input) || input === null || isPlainishObject(input);
}
//# sourceMappingURL=getGraphQLAPIs.js.map
