import {ObjectSchemaType} from '@sanity/types'
import {Observable} from 'rxjs'
import {SanityClient} from '@sanity/client'
import {SanityDocumentLike} from '@sanity/types'
import {Schema} from '@sanity/types'

/** @internal */
export declare const createSearch: SearchStrategyFactory<TextSearchResults | WeightedSearchResults>

/**
 * @internal
 */
export declare const getSearchableTypes: (schema: Schema) => ObjectSchemaType[]

/**
 * @internal
 */
export declare function getSearchTypesWithMaxDepth(
  types: ObjectSchemaType[],
  maxFieldDepth?: number,
): SearchableType[]

/**
 * @internal
 */
declare interface SearchableType {
  name: string
  title?: string
  __experimental_search: ObjectSchemaType['__experimental_search']
}

/**
 * @internal
 */
declare interface SearchFactoryOptions {
  filter?: string
  params?: Record<string, unknown>
  tag?: string
  unique?: boolean
  unstable_enableNewSearch?: boolean
}

/**
 * @internal
 */
declare interface SearchHit {
  hit: SanityDocumentLike
}

/**
 * @internal
 */
declare type SearchOptions = {
  __unstable_extendedProjection?: string
  comments?: string[]
  includeDrafts?: boolean
  skipSortByScore?: boolean
  sort?: SearchSort[]
  cursor?: string
  limit?: number
}

/**
 * @internal
 */
declare type SearchSort = {
  direction: SortDirection
  field: string
  mapWith?: string
}

/**
 * @internal
 */
declare interface SearchStory {
  path: string
  score: number
  why: string
}

/**
 * @internal
 */
declare type SearchStrategyFactory<TResult extends TextSearchResults | WeightedSearchResults> = (
  types: SearchableType[],
  client: SanityClient,
  commonOpts: SearchFactoryOptions,
) => (searchTerms: string | SearchTerms, searchOpts?: SearchOptions) => Observable<TResult>

/**
 * @internal
 */
declare interface SearchTerms {
  filter?: string
  params?: Record<string, unknown>
  query: string
  types: SearchableType[]
}

/**
 * @internal
 */
declare type SortDirection = 'asc' | 'desc'

/**
 * @internal
 */
declare interface TextSearchResults {
  type: 'text'
  hits: SearchHit[]
  nextCursor?: string
}

/** @internal */
export declare function useSearchMaxFieldDepth(overrideClient?: SanityClient): number

/**
 * @internal
 */
declare interface WeightedHit extends SearchHit {
  resultIndex: number
  score: number
  stories: SearchStory[]
}

/**
 * @internal
 */
declare interface WeightedSearchResults {
  type: 'weighted'
  hits: WeightedHit[]
  nextCursor?: never
}

export {}
